#
# $Id: Newsletter.py 108 2006-02-09 09:57:25Z zope $
#

"""Newsletter class"""

# Python core imports
import string
import commands
import cStringIO, MimeWriter, quopri
from email import Header

# Zope core imports
from Globals import InitializeClass
from AccessControl import getSecurityManager, ClassSecurityInfo, Unauthorized
from AccessControl.SpecialUsers import nobody
from Products.PageTemplates import Expressions
from Products.PythonScripts.standard import url_quote
from DateTime import DateTime
from OFS import Folder
from DocumentTemplate.DT_Util import html_quote

## Use MaildropHost, if available
maildrop_host_is_available = 0
try:
    import Products.MaildropHost
    maildrop_host_is_available = 1
except:
    pass

# CMF/Plone imports
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFDefault.SkinnedFolder import SkinnedFolder
from Products.CMFPlone.PloneFolder import OrderedContainer
from Products.CMFCore.utils import getToolByName, format_stx

try:
    from OFS.IOrderSupport import IOrderContainer as IZopeOrderedContainer
    hasZopeOrderedSupport=1
except ImportError:
    hasZopeOrderedSupport=0
from Products.CMFPlone.interfaces.OrderedContainer import IOrderedContainer

# Application level imports

from PNLPermissions import *
from PNLUtils import *
from PNLBase import PNLContentBase

#################
## The factory ##
#################

def addNewsletter(self, id, title = '', REQUEST = {}):
    """
    Factory method for a Newsletter object
    """
    obj = Newsletter(id, title)
    self._setObject(id, obj)
    getattr(self, id)._post_init()
    if REQUEST.has_key('RESPONSE'):
        return REQUEST.RESPONSE.redirect(self.absolute_url() + '/manage_main')

#################################
## The Newsletter content type ##
#################################

class Newsletter(SkinnedFolder, OrderedContainer, DefaultDublinCoreImpl, PNLContentBase):
    """Newsletter class"""
    
    if hasZopeOrderedSupport:
        __implements__ = (IOrderedContainer, IZopeOrderedContainer)
    else:
        __implements__ = (IOrderedContainer,)

    ########################################
    ## Registration info for portal_types ##
    ########################################

    factory_type_information = {
        'id': 'Newsletter',
        'portal_type': 'Newsletter',
        'meta_type': 'Newsletter',
        'description': 'A newletter (has no sense oudside a NewsletterTheme object)',
        'content_icon': 'Newsletter.gif',
        'product': 'PloneGazette',
        'factory': 'addNewsletter',
        'immediate_view': 'folder_listing',
        'global_allow': 0,
        'filter_content_types': 1,
        'allowed_content_types': ('Section', 'NewsletterTopic', 'NewsletterReference'),
        'actions': (
            {
                'id': 'view',
                'name': 'View',
                'action': 'string:${object_url}/Newsletter_view',
                'permissions': (View, ),
                'category': 'object'
                },
            {
                'id' : 'folderlisting',
                'name' : 'Folder Listing',
                'action' : 'string:${object_url}/folder_contents',
                'permissions' : (ChangeNewsletter,),
                'category' : 'object'
                },
            {
                'id': 'edit',
                'name': 'Edit',
                'action': 'string:${object_url}/Newsletter_editForm',
                'permissions': (ChangeNewsletter,),
                'category': 'object'
                },
            {
                'id': 'test',
                'name': 'Test',
                'action': 'string:${object_url}/Newsletter_testForm',
                'permissions': (ChangeNewsletter,),
                'category': 'object'
                },
            
            {
                'id': 'send',
                'name': 'Send',
                'action': 'string:${object_url}/Newsletter_sendForm',
                'permissions': (ChangeNewsletter,),
                'condition': 'python: object.canBeSent()', # Requires Plone 2.x
                'category': 'object'
                },
            {
                'id': 'metadata',
                'name': 'Properties',
                'action': 'string:${object_url}/metadata_edit_form',
                'permissions': (ChangeNewsletter,),
                'category': 'object'
                },
            ),
        }
    
    ###########################
    ## Basic class behaviour ##
    ###########################

    meta_type = factory_type_information['meta_type']
    manage_options = Folder.Folder.manage_options

    # Standard security settings
    security = ClassSecurityInfo()
    security.declareObjectProtected(View)
    security.declareProtected(ChangeNewsletter, "dummyMethod_editPermission")

    _stx_level = 1
    cooked_text = text = text_format = ''

    # Init method
    security.declarePrivate('__init__')
    def __init__(self, id, title='', description='', text_format='', text='', dateEmitted=None):
        """__init__(self, id, title='')"""

        DefaultDublinCoreImpl.__init__(self)
        self.id = id
        self.title = title
        self.description = description
        self._edit(text=text, text_format=text_format)
        self.setFormat(text_format)
        self.dateEmitted = dateEmitted
        return

    security.declarePrivate('_post_init')
    def _post_init(self):
        """
        _post_init(self) => Post-init method (that is, method that is called AFTER the class has been set into the ZODB)
        """

        self.indexObject()
        return

    #############################
    ## Content editing methods ##
    #############################

    def _edit(self, text, text_format=''):
        """
        """
        level = self._stx_level
        if not text_format:
            text_format = self.text_format

        if self.text_format:
            self.text_format = text_format
        if text_format == 'html':
            self.text = self.cooked_text = text
        elif text_format == 'plain':
            self.text = text
            self.cooked_text = html_quote(text).replace('\n', '<br />')
        else:
            self.cooked_text = format_stx(text=text, level=level)
            self.text = text

    # Edit method (change this to suit your needs)
    # This edit method should only change attributes that are neither 'id' or metadatas.
    security.declareProtected(ChangeNewsletter, 'edit')
    def edit(self, title='', text='', dateEmitted=None, text_format=''):
        """
        edit(self, text = '') => object modification method
        """
        level = self._stx_level
        # Change attributes
        if title:
            self.title = title
        try:
            self.dateEmitted = DateTime(dateEmitted)
        except:
            self.dateEmitted = None

        self.setFormat(text_format)
        self._edit(text=text, text_format=text_format)

        # Reindex
        self.reindexObject()
        return

    security.declareProtected(View, 'CookedBody')
    def CookedBody(self, stx_level=None, setLevel=0):
        """
        """
        if (self.text_format == 'html' or self.text_format == 'plain'
            or (stx_level is None)
            or (stx_level == self._stx_level)):
            return self.cooked_text
        else:
            cooked = format_stx(self.text, stx_level)
            if setLevel:
                self._stx_level = stx_level
                self.cooked_text = cooked
            return cooked

    security.declareProtected(View, 'Format')
    def Format(self):
        """
        """
        if self.text_format == 'html':
            return 'text/html'
        else:
            return 'text/plain'

    security.declareProtected(ModifyPortalContent, 'setFormat')
    def setFormat(self, format):
        """
        """
        value = str(format)
        if value == 'text/html' or value == 'html':
            self.text_format = 'html'
        elif value == 'text/plain':
            if self.text_format not in ('structured-text', 'plain'):
                self.text_format = 'structured-text'
        elif value =='plain':
            self.text_format = 'plain'
        else:
            self.text_format = 'structured-text'

    ############################
    ## portal_catalog support ##
    ############################

    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        "Returns a concatination of all searchable text"

        ret="%s %s %s" % (self.Title(),
                          self.Description(),
                          self.text)
        return ret

    ###################


    security.declarePublic('canBeSent')
    def canBeSent(self):
        """Because of Lynx (can't provide owner authentication)
        we can only send publicly available newsletters"""
        
        if nobody.has_permission('View', self):
            return 1
        return 0
        
    security.declarePublic('testMUAHtml')
    def testMUAHtml(self, REQUEST=None):
        """Makes the HTML part for MUA of the newsletter"""

        if not REQUEST:
            REQUEST = self.REQUEST
        theme = self.getTheme()
        template = theme.getRenderTemplate()
        newsletterFooter = theme.newsletterFooter
        if newsletterFooter.find('%(url)s') != -1:            
            newsletterFooter = theme.newsletterFooter.replace('%(url)s',  theme.absolute_url() + '/xxx')
            
        # Modification par jerome : customisation de Newsletter_testForm pour ajouter une query_string
        # quand on veut avoir la prévisualisation en html.
        if REQUEST.has_key('html'):
            query_html = 1
        else:
            query_html = 0
        data = template(id=self.id, body=self.cooked_text, description=self.description, newsletterFooter=newsletterFooter, html=query_html, title=self.title, date=self.dateEmitted)
        REQUEST.RESPONSE.setHeader('Content-Type', 'text/html; charset=%s' % self.ploneCharset())
        return data

    def _runLynx(self, url, charset = None):
        """Run Lynx on an url"""
        # get the good lynx command        
        theme = self.getTheme()
        lynxcommand = theme.lynxCommand
        if not lynxcommand:
            lynxcommand = LYNX_COMMAND
        # get the good lynx options        
        try:
            lynxoptions = theme.lynxOptions
        except:
            lynxoptions = None
        if not lynxoptions:
            lynxoptions = LYNX_OPTIONS

        if (charset) and ("lynx" in lynxcommand):
            lynxoptions += " -display_charset=%s " % charset

        return commands.getstatusoutput('%s %s %s' % (lynxcommand, lynxoptions, url))
    
    security.declarePublic('testMUATextPlain')
    def testMUATextPlain(self, REQUEST=None):
        """Makes the text/plain part for MUA of the newsletter"""
        
        if not REQUEST:
            REQUEST = self.REQUEST
        nlUrl = self.absolute_url() + '/testMUAHtml'
        charset = self.ploneCharset()

        status, output = self._runLynx(charset = charset, url = nlUrl)
        REQUEST.RESPONSE.setHeader('Content-Type', 'text/plain; charset=%s' % charset)

        return output
    
    security.declareProtected(ChangeNewsletter, 'testSendToMe')
    def testSendToMe(self, REQUEST=None):
        """Sends HTML/mixed and plain text newsletter to the author"""

        translate = self.translate
        if not REQUEST:
            REQUEST = self.REQUEST
        theme = self.getTheme()
        email = theme.testEmail
        editurl = theme.absolute_url() + '/xxx'
        recipients = [(email, 'HTML', editurl), (email, 'Text', editurl)]
        errors = self.sendToRecipients(recipients)
        statusMsg = "?portal_status_message="
        if errors:
            statusMsg += url_quote(translate('SMTP server related errors sending to ', domain='plonegazette')) + email
        else:
            statusMsg += url_quote(translate('Watch your mail, newsletter has been sent to ', domain='plonegazette')) + email
        REQUEST.RESPONSE.redirect(REQUEST.HTTP_REFERER + statusMsg)
        return
    
    ########################
    ## Sending newsletter ##
    ########################
    
    security.declarePublic('getHTMLTemplate')
    def getHTMLTemplate(self, html=None, REQUEST=None):
        """Returns a MIME mixed message suitable to a dict formatter"""

        if not REQUEST:
            REQUEST = self.REQUEST
        theme = self.getTheme()
        template = theme.getRenderTemplate()
        newsletterFooter = theme.newsletterFooter
        if html:
            query_html = 1
        else:
            query_html = 0

        text = self.urlTransform(self.cooked_text)
        htmlTpl = template(id=self.id, body=text, description=self.description, newsletterFooter=newsletterFooter, html=query_html, title=self.title, date=self.dateEmitted)
        REQUEST.RESPONSE.setHeader('Content-Type', 'text/html; charset=%s' % self.ploneCharset())
        return escPercent(htmlTpl)
    
    security.declarePublic('getPlaintextTemplate')
    def getPlaintextTemplate(self):
        """Returns a MIME mail plain text message suitable to a dict formatter"""

        nlUrl = self.absolute_url() + '/getHTMLTemplate'
        charset = self.ploneCharset()
 
        status, output = self._runLynx(url = nlUrl, charset = charset)
        return escPercent(output)

    security.declareProtected(ChangeNewsletter, 'sendToRecipients')
    def sendToRecipients(self, recipients):
        """Send the newsletter to a list of recipients
        recipients is a list of tuples in the form:
        [(email, format, editurl),...]
        email is the mail address
        format is 'HTML' to receive HTML/mixed mail
        editurl is the user preference URL"""

        htmlTpl = self.getHTMLTemplate(html='1')
        if htmlTpl.find('%(url)s') != -1:
            hasurl = 1
        else:
            hasurl = 0
        plaintextTpl = self.getPlaintextTemplate()
        theme = self.getTheme()
        mailFrom = theme.authorEmail
        charset = self.ploneCharset()
        errors = []

        maildrop_host = getattr(self, 'MailHost', None)
        if maildrop_host_is_available and \
               maildrop_host and maildrop_host.meta_type == 'Maildrop Host':
            mailMethod = maildrop_host._send
        else:
            smtp = theme.getSmtp()
            mailMethod = smtp.sendmail

        # the header of the title is encoded with the plone charset
        # and put back to string so it can be used afterwards
        titleForMessage = str(Header.Header(self.title, charset))
        
        for email, format, editurl in recipients:
           
            mimeMsg = cStringIO.StringIO()
            writer = MimeWriter.MimeWriter(mimeMsg)
            writer.addheader('From', mailFrom)
            writer.addheader('To', email)
            if theme.replyto:
                writer.addheader('Reply-To', theme.replyto)
            writer.addheader('Subject', titleForMessage)
            writer.addheader('MIME-Version', '1.0')
            if format == 'HTML':
                new_htmlTpl = htmlTpl
                if hasurl:
                    new_htmlTpl = new_htmlTpl.replace('%(url)s', editurl)
                
                new_plaintextTpl = plaintextTpl
                if hasurl:
                    new_plaintextTpl = new_plaintextTpl.replace('%(url)s', editurl)
                # Build and send Mixed mail
                writer.flushheaders()
                writer.startmultipartbody('alternative', '--------=BoundaryEoxxzykttGrrr------')
                # Partie text/plain
                subpart = writer.nextpart()
                subpart.addheader('Content-Transfer-Encoding', 'quoted-printable')
                subflow = subpart.startbody('text/plain; charset=%s' % charset.upper())
                quopri.encode(cStringIO.StringIO(new_plaintextTpl), subflow, 1)
                # Partie text/html
                subpart = writer.nextpart()
                subpart.addheader('Content-Transfer-Encoding', 'quoted-printable')
                subflow = subpart.startbody('text/html; charset=%s' % charset.upper())
                quopri.encode(cStringIO.StringIO(new_htmlTpl), subflow, 1)
                writer.lastpart()
            else:
                new_plaintextTpl = plaintextTpl
                if hasurl:
                    new_plaintextTpl = new_plaintextTpl.replace('%(url)s', editurl)
                
                # Build plain text mail
                writer.addheader('Content-Transfer-Encoding', 'quoted-printable')
                writer.flushheaders()
                fh = writer.startbody('text/plain; charset=%s' % charset.upper())
                quopri.encode(cStringIO.StringIO(new_plaintextTpl), fh, 1)

            try:
                mailMethod(mailFrom, (email,), mimeMsg.getvalue())
            except:
                errors.append(email)
        return errors

    security.declareProtected(ChangeNewsletter, 'sendToSubscribers')
    def sendToSubscribers(self, REQUEST=None):
        """Sends that newsletter to all subscribers and extra recipients"""

        translate = self.translate
        if REQUEST is None:
            REQUEST = self.REQUEST
        if REQUEST.REQUEST_METHOD == 'POST':
            theme = self.getTheme()
            recipients = theme.mailingInfos()
            errors1 = self.sendToRecipients(recipients)
            recipients = theme.getExtraRecipients()
            errors2 = self.sendToRecipients(recipients)
            if errors1 or errors2:
                statusMsg = translate('SMTP server related errors', domain='plonegazette')
            else:
                statusMsg = translate('The newsletter has been sent', domain='plonegazette')
            REQUEST.set('portal_status_message', statusMsg)
            self.dateEmitted = DateTime()
            return self.Newsletter_sendForm(errors1=errors1, errors2=errors2)
        return []

    ###############
    ## Utilities ##
    ###############

    security.declarePublic('getObjects')
    def getObjects(self):
       """
       """
       hasPermission = nobody.has_permission
       objects = [object for object in self.objectValues(('Section','NewsletterTopic', 'NewsletterReference')) if hasPermission('View', object)]
       objects.sort(lambda a,b:cmp(self.getObjectPosition(a.getId()), self.getObjectPosition(b.getId())))
       return objects

# Class instanciation
InitializeClass(Newsletter)

